package applications;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.util.Hashtable;


/**
 * An entity that holds the default language properties, for example 
 * all label names in English. 
 **/
public abstract class BrickTextureLanguageProperties {
  
  private static final String KEY_PREFIX = "BrickTexture.";
  
  
  public static final String KEY_BRICK_COLOR = KEY_PREFIX + "BrickColor";
  public static final String KEY_BRICK_BORDER_COLOR = KEY_PREFIX + "BrickBorderColor";
  public static final String KEY_DRAW_BORDER = KEY_PREFIX + "DrawBorder";
  public static final String KEY_BEVEL_MODE = KEY_PREFIX + "BevelMode";
  public static final String KEY_RANDOM_MODE = KEY_PREFIX + "RandomMode";
  public static final String KEY_BRICK_COL_RAND_MAGN = KEY_PREFIX + "BrickColorRandomMagnitude";
  public static final String KEY_BRICK_WIDTH = KEY_PREFIX + "BrickWidth";
  public static final String KEY_BRICK_HEIGHT = KEY_PREFIX + "BrickHeight";
    
  private static final String KEY_BEVEL_PREFIX = "BevelMode.";
  public static final String KEY_BEVEL_MODE_NONE = KEY_PREFIX + KEY_BEVEL_PREFIX + "None";
  public static final String KEY_BEVEL_MODE_RAISED = KEY_PREFIX + KEY_BEVEL_PREFIX + "Raised";
  public static final String KEY_BEVEL_MODE_SUNK = KEY_PREFIX + KEY_BEVEL_PREFIX + "Sunk";
    
  private static final String KEY_RANDOM_MODE_PREFIX = "RandomMode.";
  public static final String KEY_RANDOM_MODE_RGB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "RGB";
  public static final String KEY_RANDOM_MODE_R = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Red";
  public static final String KEY_RANDOM_MODE_G = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Green";
  public static final String KEY_RANDOM_MODE_B = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Blue";
  public static final String KEY_RANDOM_MODE_RG = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "RG";
  public static final String KEY_RANDOM_MODE_RB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "RB";
  public static final String KEY_RANDOM_MODE_GB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "GB";
  public static final String KEY_RANDOM_MODE_HSB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "HSB";
  public static final String KEY_RANDOM_MODE_H = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Hue";
  public static final String KEY_RANDOM_MODE_S = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Saturation";
  public static final String KEY_RANDOM_MODE_BR = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "Brightness";
  public static final String KEY_RANDOM_MODE_HS = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "HS";
  public static final String KEY_RANDOM_MODE_HB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "HB";
  public static final String KEY_RANDOM_MODE_SB = KEY_PREFIX + KEY_RANDOM_MODE_PREFIX + "SB";
  
  
  public static void addDefaultLanguageProperties(Hashtable<String, String> pProperties) {
    if (pProperties == null) return;
    
    pProperties.put(KEY_BRICK_COLOR, "Brick color");
    pProperties.put(KEY_BRICK_BORDER_COLOR, "Brick border color");
    pProperties.put(KEY_DRAW_BORDER, "Draw brick border");
    pProperties.put(KEY_BEVEL_MODE, "Bevel mode");
    pProperties.put(KEY_RANDOM_MODE, "Random mode");
    pProperties.put(KEY_BRICK_COL_RAND_MAGN, "Randomize magnitude");
    pProperties.put(KEY_BRICK_WIDTH, "Brick width");
    pProperties.put(KEY_BRICK_HEIGHT, "Brick height");
    
    pProperties.put(KEY_BEVEL_MODE_NONE, "None");
    pProperties.put(KEY_BEVEL_MODE_RAISED, "Raised");
    pProperties.put(KEY_BEVEL_MODE_SUNK, "Sunk");
    
    pProperties.put(KEY_RANDOM_MODE_RGB, "RGB");
    pProperties.put(KEY_RANDOM_MODE_R, "Red");
    pProperties.put(KEY_RANDOM_MODE_G, "Green");
    pProperties.put(KEY_RANDOM_MODE_B, "Blue");
    pProperties.put(KEY_RANDOM_MODE_RG, "Red - Green");
    pProperties.put(KEY_RANDOM_MODE_RB, "Red - Blue");
    pProperties.put(KEY_RANDOM_MODE_GB, "Green - Blue");
    pProperties.put(KEY_RANDOM_MODE_HSB, "HSB");
    pProperties.put(KEY_RANDOM_MODE_H, "Hue");
    pProperties.put(KEY_RANDOM_MODE_S, "Saturation");
    pProperties.put(KEY_RANDOM_MODE_BR, "Brightness");
    pProperties.put(KEY_RANDOM_MODE_HS, "Hue - Saturation");
    pProperties.put(KEY_RANDOM_MODE_HB, "Hue - Brightness");
    pProperties.put(KEY_RANDOM_MODE_SB, "Saturation - Brightness");
  }
}