package applications;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;



import javax.swing.JPanel;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.BoxLayout;
import javax.swing.JFrame;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


import gui.ImageEffectSaveOption;
import static applications.LanguageProperties.*;
import static applications.Settings.*;

/***
 *   Dialog that represents the options of the gui application.
 **/

public class SaveOptionsDialog extends JDialog implements ActionListener {

  /*** member variables ***/
  private Vector<ImageEffectSaveOption> m_SaveOptions;
  private Vector<JCheckBox> m_CheckBoxes;
  private boolean m_bAborted;

  /*** gui member variables ***/
  private JButton m_OkButton, m_AbortButton;


  /////////////////////////////////
  // Constructor                 //
  /////////////////////////////////
  /**
   *  TODO: Creates the settings dialog.
   *
   *  @param pOwner The {@code JFrame} object that is blocked while the settings dialog is shown.
   *
   *  @throws Exception If the application settings could not be loaded.
   **/
  public SaveOptionsDialog(JFrame pOwner, Vector<ImageEffectSaveOption> saveOptions) {
    //1. init the dialog and make a check
    super(pOwner);
    setModal(true);

    m_SaveOptions = saveOptions;
    m_bAborted = false;

    createDialog();

    /*
    //2. get the settings
    m_pSettings = getSettings();
    LanguageProperties pLanguageProperties = m_pSettings.getLanguageProperties();

    //3. set the layout
    setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

    JPanel pLanguagePanel = new JPanel();
    pLanguagePanel.setLayout(new FlowLayout());
    m_pLanguageLabel = new JLabel(pLanguageProperties.getProperty(LANG_KEY_LANGUAGE));
    pLanguagePanel.add(m_pLanguageLabel);

    Vector<String> pLanguages = new Vector<String>();
    pLanguages.add("English");
    pLanguages.add("German");
    pLanguages.add("Austrian");
    m_pLanguageCombo = new JComboBox<String>(pLanguages);
    m_pLanguageCombo.setSelectedItem(m_pSettings.getProperty(PROP_KEY_LANGUAGE));
    pLanguagePanel.add(m_pLanguageCombo);




    //4. ok and abort button
    JPanel pOkPanel = new JPanel();
    pOkPanel.setLayout(new FlowLayout());

    m_pOk = new JButton(pLanguageProperties.getProperty(KEY_OK));
    m_pOk.addActionListener(this);
    pOkPanel.add(m_pOk);

    m_pAbort = new JButton(pLanguageProperties.getProperty(KEY_ABORT));
    m_pAbort.addActionListener(this);
    pOkPanel.add(m_pAbort);

    //5. add the panels
    add(pLanguagePanel);
    add(pOkPanel);




    pack(); // sets the optimum size to the dialog

    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iXPos = (pScreenSize.width-getWidth())/2;
    int iYPos = (pScreenSize.height-getHeight())/2;
    setLocation(iXPos, iYPos);
    */

  }


  ///////////////////////////////////
  // GET METHODS                   //
  ///////////////////////////////////


  public boolean shouldBeShown() {
    if (m_SaveOptions == null) return false;
    if (m_SaveOptions.size() == 0) return false;
    return true;
  }

  public boolean aborted() {
    return m_bAborted;
  }

  public Vector<ImageEffectSaveOption> getSaveOptions() {
    return m_SaveOptions;
  }




  ////////////////////////////////////
  // SET METHODS                    //
  ////////////////////////////////////




  ////////////////////////////////////////
  // ACTIONLISTENER METHODS             //
  ////////////////////////////////////////

  /**
   *  Implementation of the {@code ActionListener} interface to handle button events.
   *
   *  @param e The {@code ActionEvent} object that is used to get the source of the action.
   **/
  public void actionPerformed(ActionEvent e) {
    Object pSource = e.getSource();




    //save the settings, apply all settings to the listeners, close the dialog
    if (pSource.equals(m_OkButton)) {
      applyCheckBoxSettings();
      m_bAborted = false;
    }
    else {
      m_bAborted = true;
    }

    setVisible(false);


  }



  //////////////////////////////////////
  // PRIVATE HELPER METHODS           //
  //////////////////////////////////////
  private void createDialog() {
    setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));

    LanguageProperties languageProperties = SettingsDialog.getSettings().getLanguageProperties();
    setTitle(languageProperties.getProperty(KEY_SAVE_OPTIONS_DIALOG_TITLE));

    createSaveOptions(languageProperties);

    createSaveButtons(languageProperties);

    pack(); // sets the optimum size to the dialog

    //TODO: DialogHelpers.getLocation(getWidth(), getHeight());
    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iXPos = (pScreenSize.width-getWidth())/2;
    int iYPos = (pScreenSize.height-getHeight())/2;
    setLocation(iXPos, iYPos);
  }

  private void createSaveOptions(LanguageProperties languageProperties) {
    if (!shouldBeShown()) return;

    m_CheckBoxes = new Vector<JCheckBox>();
    int iSaveOptionNum = m_SaveOptions.size();
    for (int i = 0; i < iSaveOptionNum; i++) {
      ImageEffectSaveOption saveOption = m_SaveOptions.get(i);
      JCheckBox checkBox = new JCheckBox(saveOption.getLabel(languageProperties), saveOption.isSelected());
      add(checkBox);
      m_CheckBoxes.add(checkBox);
    }


  }




  private void createSaveButtons(LanguageProperties languageProperties) {
    JPanel panel = new JPanel();
    panel.setLayout(new FlowLayout());

    m_OkButton = new JButton(languageProperties.getProperty(KEY_OK));
    m_OkButton.addActionListener(this);
    panel.add(m_OkButton);

    m_AbortButton = new JButton(languageProperties.getProperty(KEY_ABORT));
    m_AbortButton.addActionListener(this);
    panel.add(m_AbortButton);

    //5. add the panels
    add(panel);
  }

  private void applyCheckBoxSettings() {
    if (!shouldBeShown()) return;

    int iSaveOptionNum = m_SaveOptions.size();
    for (int i = 0; i < iSaveOptionNum; i++) {
      m_SaveOptions.get(i).setSelected(m_CheckBoxes.get(i).isSelected());
    }

  }






}
