package fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.awt.Color;
import java.awt.image.BufferedImage;

//library imports
import standard.EffectParameters;
import util.color.ColorGradient;
import util.math.ComplexNumber;

import static fractals.MandelbrotSetParameters.*;

/**
 *  Implements the Mandelbrot set effect.
 **/
public class JuliaSet extends FractalSetEffect {


  
  //////////////////////////////
  // CONSTRUCTOR              //
  //////////////////////////////
  /**
   *   Creates a {@code MandelbrotSet} object with the default parameters.
   *
   **/
  public JuliaSet() {
    this(new JuliaSetParameters());
  }
  

  /**
   *   Creates a {@code MandelbrotSet} object that draws a part of the mandelbrot set according to the specified parameters.
   *
   *   @param pParameters The parameters that configure the Mandelbrot set image.
   **/
  public JuliaSet(JuliaSetParameters pParameters) {
    //1. Make an initial check
    if (pParameters == null) pParameters = new JuliaSetParameters();
    
    //2. set the member variables
    setFractalSetParameters(pParameters);
  }
  

  

}
