package gui.color;

/*
 *  Copyright (C) 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.swing.JPanel;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/**
 *  Shows the ColorChooser as it should be in the JFXLibrary.
 **/
public class ColorChooser implements ActionListener {
  
  /** private gui member variables **/
  private JColorChooser m_pColorChooser;
  private IColorChooserListener m_pListener;
  
  //////////////////////////
  // CONSTRUCTOR          //
  //////////////////////////
  /**
   *  Creates a default {@code ColorDisplay} object with the color Black and an empty title for the Color chooser dialog.
   **/
  public ColorChooser(IColorChooserListener pListener) {
    //original call
    //Color pNewColor = JColorChooser.showDialog(this, m_pColorChooserTitle, m_pColor); 
    m_pColorChooser = new JColorChooser();
    //TODO: set layout
    m_pListener = pListener;
  }
  
  
  //////////////////////////
  // SHOW METHODS       //
  //////////////////////////
  public void showDialog(JPanel pPanel, String pTitle, Color pColor) {
    JColorChooser pChooser = new JColorChooser(pColor);
    
   
   
    if (pColor != null) m_pColorChooser.setColor(pColor);
    JDialog pDialog = JColorChooser.createDialog(pPanel, pTitle, true, m_pColorChooser, this, null);
    
    Dimension pScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
    int iXPos = (pScreenSize.width-pDialog.getWidth())/2;
    int iYPos = (pScreenSize.height-pDialog.getHeight())/2;
    pDialog.setLocation(iXPos, iYPos);
    //pDialog.setDragEnabled(true);
    pDialog.setVisible(true);
    
    
    
    
    
    
  }
  
  
  //////////////////////////////
  // ACTION LISTENER          //
  //////////////////////////////
  public void actionPerformed(ActionEvent e) {
    if (m_pListener == null) return;
    m_pListener.setColor(m_pColorChooser.getColor());
  }
  
  
  
  
  
}