package gui.fractals;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */


import java.util.Vector;
import javax.swing.JPanel;
import java.awt.FlowLayout;
import javax.swing.event.ChangeEvent;

//Library imports
import fractals.JuliaSetParameters;
import gui.DoubleSliderPanel;
import gui.SliderPanel;
import applications.LanguageProperties;

import static applications.FractalSetLanguageProperties.*;



/**
 *  A properties panel for the mandelbrot effect.
 **/
public class JuliaSetPropertiesPanel extends FractalSetPropertiesPanel implements IJuliaSetParameterCPanelListener {

  /*** member variables ***/
  private JuliaSetParameterCPanel m_pParameterCPanel;
  private JuliaSetPanel m_pJuliaSetPanel;


  ////////////////////////////////
  // CONSTRUCTOR                //
  ////////////////////////////////
  /**
   *  Creates a {@code JuliaSetPropertiesPanel} object with the parameters that are set to a given  {@code JuliaSetPanel}.
   *
   *  @param pMandelbrotSetPanel The {@code JuliaSetPanel} object from which the effect properties are taken.
   **/
  public JuliaSetPropertiesPanel(JuliaSetPanel pJuliaSetPanel) {
    //1. init panel
    m_pJuliaSetPanel = pJuliaSetPanel;
    m_pParameterCPanel = new JuliaSetParameterCPanel();
    m_pParameterCPanel.addPanelListener(this);
    initPanel(pJuliaSetPanel);


  }


  //////////////////////////////
  // PROTECTED METHODS        //
  //////////////////////////////
  protected Vector<String> getAdditionalPropertyKeys() {
    Vector<String> pAdditionalProps = new Vector<String>();
    pAdditionalProps.add(KEY_PARAMETER_C);
    return pAdditionalProps;
  }


  protected Vector<JPanel> getAdditionalPropertyPanels() {
    Vector<JPanel> pPanels = new Vector<JPanel>();
    pPanels.add(m_pParameterCPanel);
    return pPanels;
  }

  //////////////////////////////////////////
  // JULIA SET PARAMETER C PANEL LISTENER //
  //////////////////////////////////////////
  public void setParameterC(JuliaSetParameterCPanel panel, double dReal, double dImaginary) {
    m_pJuliaSetPanel.setCParameter(dReal, dImaginary);
  }


  protected void setAdditionalLabels(LanguageProperties pLanguageProperties) {
    m_pParameterCPanel.setLabels(pLanguageProperties);
  }

  ///////////////////////////////
  // PRIVATE METHODS           //
  ///////////////////////////////





}
