package gui.fractals;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */



//library imports
import fractals.MandelbrotSet;
import fractals.MandelbrotSetParameters;
import gui.AbstractPropertiesPanel;



/**
 *  Handles the interaction with a Mandelbrot set.
 **/

public class MandelbrotSetPanel extends FractalSetPanel {

  
  
  /** member variables **/
  private MandelbrotSetPropertiesPanel m_pPropertiesPanel;
  
  
  
  ////////////////////////////////////
  // CONSTRUCTOR                    //
  ////////////////////////////////////
  /**
   *  Creates a {@code MandelbrotSetPanel} object with the default settings for the Mandelbrot set effect.
   **/
  public MandelbrotSetPanel() {
    
    m_pPropertiesPanel = null;
    
    initPanel(new MandelbrotSetParameters(),  new MandelbrotSet());
    
   
  }

  
  ///////////////////////////////////////
  // SET METHODS                       //
  ///////////////////////////////////////
 
 
  
  
  ///////////////////////////////////////
  // GET METHODS                       //
  ///////////////////////////////////////

  /**
   *  Returns the {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   *
   *  @return The {@code MandelbrotSetPropertiesPanel} object representing the effect parameters and providing a GUI to modify them.
   **/
  public AbstractPropertiesPanel getPropertiesPanel() {
    if (m_pPropertiesPanel == null) {
      m_pPropertiesPanel = new MandelbrotSetPropertiesPanel(this);
    }
    
    return m_pPropertiesPanel;
  }
  
  
 
  
  

}