package gui.image;

/*
 *  Copyright (C) 2018 Juergen Platzer
 *
 *  This file is part of JFXLibrary.
 *
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//gui imports
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import javax.imageio.ImageIO;

import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.image.BufferedImage;

import java.util.Vector;
import java.io.File;


import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

//library imports
import applications.Settings;
import applications.ISettingsListener;
import applications.SettingsDialog;
import applications.LanguageProperties;
import gui.io.ImageDisplay;
import gui.io.IImageDisplayListener;
import gui.JPanelWithListener;



/**
 *   A panel that is used to modify an image property by loading an image from an external source.
 *
 **/
public class MultipleImageLoadPanel extends JPanelWithListener<IMultipleImageLoadPanelListener>
  implements IImageLoadPanelListener, ISettingsListener {

  /*** member variables ***/
  private Vector<ImageLoadPanel> m_pImageLoadPanels;
  private int m_iModifiedImageId;


  /*** private members ***/

  ///////////////////////////////
  // CONSTRUCTOR               //
  ///////////////////////////////
  /**
   *  Creates an {@code MultipleImageLoadPanel} object with its gui components: Several {@code ImageLoadPanel} objects to load more than one image.
   *
   */
  public MultipleImageLoadPanel(int iNumberOfImages) {

    //0. get the language settings
    Settings pSettings = SettingsDialog.getSettings();
    LanguageProperties pLanguageProperties = pSettings.getLanguageProperties();

    //1. construct the gui
    //1.1. set the layout and create a box
    setLayout(new FlowLayout(FlowLayout.LEFT));

    if (iNumberOfImages < 1) iNumberOfImages = 1;
    m_pImageLoadPanels = new Vector<>(iNumberOfImages);
    for (int i = 0; i < iNumberOfImages; i++) {
      ImageLoadPanel pPanel = new ImageLoadPanel();
      pPanel.addPanelListener(this);
      m_pImageLoadPanels.add(pPanel);
      add(pPanel);
    }
    m_iModifiedImageId = 0;







  }


  ////////////////////////////////
  // SET METHODS                //
  ////////////////////////////////
  // TODO: a set image method



  ////////////////////////////////
  // GET METHODS                //
  ////////////////////////////////
  /**
   *  Returns the image that is currently loaded into this {@code ImageLoadPanel} object.
   *
   *  @return The image that is currently loaded into this {@code ImageLoadPanel} object.
   **/
   /*
  public BufferedImage getImage() {
    return m_pImage;
  }
  */

  /**
   *  Returns the file name of the image that is currently loaded into this {@code ImageLoadPanel} object.
   *
   *  @return The file name of the image that is currently loaded into this {@code ImageLoadPanel} object.
   **/
   /*
  public String getFileName() {
    return m_pFileName;
  }
  */


  ////////////////////////////////
  // IMAGE LOAD PANEL LISTENER  //
  ////////////////////////////////
  /**
   *  This method is called when the value of the {@code ImageLoadPanel} panel has been changed.
   *
   *  @param pImageLoadPanel The {@code ImageLoadPanel} to which the listener is registered and that triggered this method call.
   *  @param pImage The {@code BufferedImage} that is now set in the {@code ImageLoadPanel}.
   */
  public void applyChanges(ImageLoadPanel pImageLoadPanel, BufferedImage pImage) {
    m_iModifiedImageId = 0;
    for (int i = 0; i < m_pImageLoadPanels.size(); i++) {
      if (pImageLoadPanel.equals(m_pImageLoadPanels.get(i))) {
        m_iModifiedImageId = i;
        break;
      }
    }
    applyChanges();
  }


  ////////////////////////////////
  // LISTENER METHODS           //
  ////////////////////////////////
  protected void applyChangesToListener(IMultipleImageLoadPanelListener pListener) {

    pListener.applyChanges(this, m_iModifiedImageId, m_pImageLoadPanels.get(m_iModifiedImageId).getImage());
  }


  ////////////////////////////////
  // SETTINGS LISTENER METHODS  //
  ////////////////////////////////
  /**
   *  Applies new settings (e.g. language settings) to this gui element.
   *
   *  @param pSettings An {@code Settings} object that provides the information that should be changed.
   **/
  public void applyNewSettings(Settings pSettings) {
    //1. make an initial check and set the settings
    if (pSettings == null) return;


    //2. load new language settings
    int iImageNum = m_pImageLoadPanels.size();
    for (int i = 0; i < iImageNum; i++) {
      m_pImageLoadPanels.get(i).applyNewSettings(pSettings);
    }

    //3. set the new labels of the combobox
  }


  //////////////////////////////////
  // PRIVATE HELPER METHODS       //
  //////////////////////////////////








}
