package standard;

/*
 *  Copyright (C) 2014 - 2018 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Vector;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.JCheckBox;

import java.awt.Color;

//library imports
/*import gui.util.IntegerSliderPanel;
import gui.util.DoubleSliderPanel;
import gui.util.ColorChoosePanel;

import gui.util.InterpolationPanel;
*/


/**
 *   Abstract class implementing basic methods for the specific effect parameter classes.
 */
public abstract class EffectParameters {

  private double m_dPosition;

  private Hashtable<String, BooleanParameter> m_pBooleanParameters = null;
  private Hashtable<String, IntegerParameter> m_pIntegerParameters = null;
  private Hashtable<String, DoubleParameter> m_pDoubleParameters = null;
  private Hashtable<String, ColorParameter> m_pColorParameters = null;
  private Hashtable<String, ObjectParameter> m_pObjectParameters = null;
  
  private Hashtable<String, IntegerParameter> m_pProtectedIntegerParameters = null;
  private Hashtable<String, ObjectParameter> m_pProtectedObjectParameters = null;



  
  //////////////////////
  // SET METHODS      //
  //////////////////////
  /**
   *  Sets the boolean value of the specified {@code BooleanParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code BooleanParameter}. 
   *  @param bValue The new boolean value of the {@code BooleanParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setBooleanParameterValue(String pParameterKey, boolean bValue) {

    //1. make a simple check
    if (m_pBooleanParameters == null) return false;

    //2. try to fetch the BooleanParameter
    BooleanParameter pParameter = m_pBooleanParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(bValue);
    return true;
  }


  /**
   *  Sets the integer value of the specified {@code IntegerParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code IntegerParameter}. 
   *  @param iValue The new integer value of the {@code IntegerParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setIntegerParameterValue(String pParameterKey, int iValue) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return false;

    //2. try to fetch the BooleanParameter
    IntegerParameter pParameter = m_pIntegerParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(iValue);
    return true;
  }
  
   /**
   *  Sets the integer value of the specified protected {@code IntegerParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code IntegerParameter}. 
   *  @param iValue The new integer value of the {@code IntegerParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  protected boolean setProtectedIntegerParameterValue(String pParameterKey, int iValue) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return false;

    //2. try to fetch the BooleanParameter
    IntegerParameter pParameter = m_pProtectedIntegerParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(iValue);
    return true;
  }
  
  
  /**
   *  Sets the double value of the specified {@code DoubleParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code DoubleParameter}. 
   *  @param dValue The new double value of the {@code DoubleParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setDoubleParameterValue(String pParameterKey, double dValue) {
    //1. make a simple check
    if (m_pDoubleParameters == null) return false;

    //2. try to fetch the DoubleParameter
    DoubleParameter pParameter = m_pDoubleParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(dValue);
    return true;
  }
  
  
  
  /**
   *  Sets the color value of the specified {@code ColorParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code ColorParameter}. 
   *  @param pColor The new color value of the {@code ColorParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setColorParameterValue(String pParameterKey, Color pColor) {
    //1. make a simple check
    if (m_pColorParameters == null) return false;
    if (pColor == null) return false;

    //2. try to fetch the BooleanParameter
    ColorParameter pParameter = m_pColorParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(pColor);
    return true;
    

  }
  
  /**
   *  Sets the value of the specified {@code ObjectParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code ObjectParameter}. 
   *  @param pObject The new value of the {@code ObjectParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setObjectParameterValue(String pParameterKey, Object pObject) {
    //1. make a simple check
    if (m_pObjectParameters == null) return false;
    // object can be null!!! is this ok?
    //if (pObject == null) return false;

    //2. try to fetch the BooleanParameter
    ObjectParameter pParameter = m_pObjectParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(pObject);
    return true;
    

  }
  
  
  /**
   *  Sets the value of the specified protected {@code ObjectParameter}. This can only executed by a subclass.
   *
   *  @param pParameterKey The name that identifies the {@code ObjectParameter}. 
   *  @param pObject The new value of the {@code ObjectParameter}. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  protected boolean setProtectedObjectParameterValue(String pParameterKey, Object pObject) {
    //1. make a simple check
    if (m_pProtectedObjectParameters == null) return false;
    if (pObject == null) return false;

    //2. try to fetch the BooleanParameter
    ObjectParameter pParameter = m_pProtectedObjectParameters.get(pParameterKey);
    if (pParameter == null) return false;

    pParameter.setValue(pObject);
    return true;
    

  }
  
  
  /**
   *  Sets the value of the specified parameter. 
   *
   *  @param pParameterKey The name that identifies the parameter. 
   *  @param pValue The new value of the parameter as string. 
   *
   *  @return true, if the value could be set, false otherwise.
   **/
  public boolean setParameterValue(String pParameterKey, String pValue) {
    //1. make a simple check
    if (pParameterKey == null) return false;
    
    //2. try to fetch the parameter
    IParameter pParameter = null;
    if (m_pIntegerParameters != null) pParameter = m_pIntegerParameters.get(pParameterKey);
    if ((pParameter == null) && (m_pDoubleParameters != null)) pParameter = m_pDoubleParameters.get(pParameterKey);
    if ((pParameter == null) && (m_pBooleanParameters != null)) pParameter = m_pBooleanParameters.get(pParameterKey);
    if ((pParameter == null) && (m_pColorParameters != null)) pParameter = m_pColorParameters.get(pParameterKey);
    if ((pParameter == null) && (m_pObjectParameters != null)) pParameter = m_pObjectParameters.get(pParameterKey);
    
    if (pParameter == null) return false;
    
    //3. try to set the value 
    return pParameter.setValue(pValue);
  }



  //////////////////////////////////
  // PROTECTED SET METHODS        //
  //////////////////////////////////

  /**
   *  Sets the {@code BooleanParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code BooleanParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setBooleanParameters(Hashtable<String, BooleanParameter> pParameters) {
    if (pParameters == null) return;

    m_pBooleanParameters = pParameters;
  }

  /**
   *  Sets the {@code IntegerParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code IntegerParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setIntegerParameters(Hashtable<String, IntegerParameter> pParameters) {
    if (pParameters == null) return;

    m_pIntegerParameters = pParameters;
  }
  
  
    /**
   *  Sets the protected {@code IntegerParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code IntegerParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setProtectedIntegerParameters(Hashtable<String, IntegerParameter> pParameters) {
    if (pParameters == null) return;

    m_pProtectedIntegerParameters = pParameters;
  }
  
  
  /**
   *  Sets the {@code DoubleParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code DoubleParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setDoubleParameters(Hashtable<String, DoubleParameter> pParameters) {
    if (pParameters == null) return;

    m_pDoubleParameters = pParameters;
  }
  
  /**
   *  Sets the {@code ColorParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code ColorParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setColorParameters(Hashtable<String, ColorParameter> pParameters) {
    if (pParameters == null) return;

    m_pColorParameters = pParameters;
  }
  
  /**
   *  Sets the {@code ObjectParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code ObjectParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setObjectParameters(Hashtable<String, ObjectParameter> pParameters) {
    if (pParameters == null) return;

    m_pObjectParameters = pParameters;
  }


  /**
   *  Sets the protected {@code ObjectParameter} objects used by this effect parameters object. This should only be called by an actual 
   *  effect parameters class. 
   *
   *  @param pParameters The {@code ObjectParameter} objects in a hashtable with s string key that is assigned to each parameter.
   **/
  protected void setProtectedObjectParameters(Hashtable<String, ObjectParameter> pParameters) {
    if (pParameters == null) return;

    m_pProtectedObjectParameters = pParameters;
  }



  //////////////////////
  // GET METHODS      //
  //////////////////////
  
  //////////
  // GET VALUES
  
  /**
   *  Returns the boolean value of the specified {@code BooleanParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code BooleanParameter}. 
   *
   *  @return The boolean value of the specified {@code BooleanParameter}. 
   **/
  public boolean getBooleanParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pBooleanParameters == null) return false;

    //2. try to fetch the BooleanParameter
    BooleanParameter pParameter = m_pBooleanParameters.get(pParameterKey);
    if (pParameter == null) return false;

    return pParameter.getValue();

  }

  /**
   *  Returns the integer value of the specified {@code IntegerParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code IntegerParameter}. 
   *
   *  @return The integer value of the specified {@code IntegerParameter}. 
   **/
  public int getIntegerParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return 0;

    //2. try to fetch the IntegerParameter
    IntegerParameter pParameter = m_pIntegerParameters.get(pParameterKey);
    if (pParameter == null) return 0;

    return pParameter.getValue();

  }
  
  
  /**
   *  Returns the integer value of the specified protected {@code IntegerParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code IntegerParameter}. 
   *
   *  @return The integer value of the specified {@code IntegerParameter}. 
   **/
  protected int getProtectedIntegerParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return 0;

    //2. try to fetch the IntegerParameter
    IntegerParameter pParameter = m_pProtectedIntegerParameters.get(pParameterKey);
    if (pParameter == null) return 0;

    return pParameter.getValue();

  }
  
  /**
   *  Returns the double value of the specified {@code DoubleParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code DoubleParameter}. 
   *
   *  @return The double value of the specified {@code DoubleParameter}. 
   **/
  public double getDoubleParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pDoubleParameters == null) return 0;

    //2. try to fetch the DoubleParameter
    DoubleParameter pParameter = m_pDoubleParameters.get(pParameterKey);
    if (pParameter == null) return 0;

    return pParameter.getValue();

  }

  /**
   *  Returns the color value of the specified {@code ColorParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code ColorParameter}. 
   *
   *  @return The color value of the specified {@code ColorParameter}. 
   **/
  public Color getColorParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pColorParameters == null) return null;

    //2. try to fetch the ColorParameter
    ColorParameter pParameter = m_pColorParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getValue();

  }
  
  /**
   *  Returns the value of the specified {@code ObjectParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code ObjectParameter}. 
   *
   *  @return The value of the specified {@code ObjectParameter}. 
   **/
  public Object getObjectParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pObjectParameters == null) return null;

    //2. try to fetch the ObjectParameter
    ObjectParameter pParameter = m_pObjectParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getValue();

  }


  
  /**
   *  Returns the value of the specified protected {@code ObjectParameter}. 
   *
   *  @param pParameterKey The name that identifies the {@code ObjectParameter}. 
   *
   *  @return The value of the specified {@code ObjectParameter}. 
   **/
  protected Object getProtectedObjectParameterValue(String pParameterKey) {
    //1. make a simple check
    if (m_pObjectParameters == null) return null;

    //2. try to fetch the ObjectParameter
    ObjectParameter pParameter = m_pProtectedObjectParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getValue();

  }

  
  
  
  /////////////////
  // GET PARAMETER NAMES
  
  /**
   *  Returns the names {@code BooleanParameter} object. 
   *
   *  @return The names {@code BooleanParameter} object as an enumeration of strings.
   **/
  public Enumeration<String> getBooleanParameterNames() {
    if (m_pBooleanParameters == null) return null;
    
    return m_pBooleanParameters.keys();
  }
  
  
  /**
   *  Returns the names {@code IntegerParameter} object. 
   *
   *  @return The names {@code IntegerParameter} object as an enumeration of strings.
   **/
  public Enumeration<String> getIntegerParameterNames() {
    if (m_pIntegerParameters == null) return null;
    
    return m_pIntegerParameters.keys();
  }
  
  /**
   *  Returns the names {@code DoubleParameter} object. 
   *
   *  @return The names {@code DoubleParameter} object as an enumeration of strings.
   **/
  public Enumeration<String> getDoubleParameterNames() {
    if (m_pDoubleParameters == null) return null;
    
    return m_pDoubleParameters.keys();
  }
  
  
  /**
   *  Returns the names {@code ColorParameter} object. 
   *
   *  @return The names {@code ColorParameter} object as an enumeration of strings.
   **/
  public Enumeration<String> getColorParameterNames() {
    if (m_pColorParameters == null) return null;
    
    return m_pColorParameters.keys();
  }
  
  /**
   *  Returns the names {@code ObjectParameter} object. 
   *
   *  @return The names {@code ObjectParameter} object as an enumeration of strings.
   **/
  public Enumeration<String> getObjectParameterNames() {
    if (m_pObjectParameters == null) return null;
    
    return m_pObjectParameters.keys();
  }
  
  
  
  

  //////////
  // GET IMAGE EFFECT GUI ELEMENT
  
  /*
  public JCheckBox getBooleanParameterGUIElement(String pParameterKey, String pLabelText) {
    //1. make a simple check
    if (m_pBooleanParameters == null) return null;

    //2. try to fetch the BooleanParameter
    BooleanParameter pParameter = m_pBooleanParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getImageEffectGUIElement(pLabelText);
  }



  public IntegerSliderPanel getIntegerParameterGUIElement(String pParameterKey, String pLabelText) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return null;

    //2. try to fetch the IntegerParameter
    IntegerParameter pParameter = m_pIntegerParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getImageEffectGUIElement(pLabelText);
  }
  
  public DoubleSliderPanel getDoubleParameterGUIElement(String pParameterKey, String pLabelText) {
    //1. make a simple check
    if (m_pDoubleParameters == null) return null;

    //2. try to fetch the DoubleParameter
    DoubleParameter pParameter = m_pDoubleParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getImageEffectGUIElement(pLabelText);
  }
  
  
  
  public ColorChoosePanel getColorParameterGUIElement(String pParameterKey, String pLabelText) {
    //1. make a simple check
    if (m_pColorParameters == null) return null;

    //2. try to fetch the ColorParameter
    ColorParameter pParameter = m_pColorParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getImageEffectGUIElement(pLabelText);
  }



  //////////
  // GET VIDEO EFFECT GUI ELEMENT
  
  public InterpolationPanel getIntegerParameterVideoEffectGUIElement(String pParameterKey, String pLabelText) {
    //1. make a simple check
    if (m_pIntegerParameters == null) return null;

    //2. try to fetch the IntegerParameter
    IntegerParameter pParameter = m_pIntegerParameters.get(pParameterKey);
    if (pParameter == null) return null;

    return pParameter.getVideoEffectGUIElement(pLabelText);
  }

 */



  ///////////////////////////////////
  // ABSTRACT METHODS              //
  ///////////////////////////////////

  /**
   *   Returns an intermediate Keyframe between the this <code>Keyframe</code> object and the <code>Keyframe</code>
   *   object given as parameter. If the given parameter is not of the same type as this <code>Keyframe</code> object
   *   the parameter is returned.
   *
   *   @param pKeyframe    The <code>Keyframe</code> object that should be used to create an intermediate <code>Keyframe</code>
   *                       object by interpolating their parameter values.
   *   @param dCurrentTime The current time for which an intermediate keyframe should be calculated as <code>double</code>..
   *
   *   @return The interpolated <code>Keyframe</code> object, if the given method parameter is of the same type as the
   *           this <code>Keyframe</code> object. If not, the parameter is returned.
   *
   **/
  //public abstract <T extends EffectParameters> T getIntermediateKeyframe(T pKeyframe, double dCurrentTime);




  //////////////////////////////////////////////
  // STATIC UTILITY METHODS FOR INTERPOLATION //
  //////////////////////////////////////////////


  /**
   *  Writes the two Keyframes that surround the given time position into the predefined Keyframe array.
   *  This utility routine can be applied to any class extending the Keyframe class. On the given <code>Vector</code> object
   *  of Keyframes a sorting operation is applied. If the second entry of the Keyframe array holds <code>null</code> then the
   *  given time position is only influenced by one Keyframe object. A boolean flag is returned indicating, if the search for
   *  the neighbouring Keyframe objects was successful.
   *
   *  @param pResultKeyframes An array of the specified keyframe class into which the neighbouring keyframes are written. Thus
   *                          it must have the length 2.
   *  @param pKeyframes       A Vector holding at least one object of the specified keyframe class. It is assumed, that the
   *                          keyframe objects are already sorted. The two keyframes that surround the given time position
   *                          are chosen from this Vector object.
   *  @param dTime            The time position as double in the [0,1] interval.
   *
   *  @return <code>true</code> if the neighbouring keyframes were found successfully.
   */
  /*public static <T extends EffectParameters> boolean findNeighbouringKeyframes(T[] pResultKeyframes, Vector<T> pKeyframes, double dTime) {

    //1. make some checks
    if (pResultKeyframes == null) return false;
    if (pResultKeyframes.length < 2) return false;
    if (pKeyframes == null) return false;
    if (pKeyframes.size() == 0) return false;


    //2. case if only one keyframe is given
    if (pKeyframes.size() == 1) {
      pResultKeyframes[0] = pKeyframes.get(0);
      pResultKeyframes[1] = null;
      return true;
    }

    //3. sort keyframes
    Collections.sort(pKeyframes);



    //4. find the keyframes
    for (int i = 0; i < pKeyframes.size(); i++) {

      T pCurrentKeyframe = pKeyframes.get(i);

      if (pCurrentKeyframe.getPosition() >= dTime) {
        pResultKeyframes[0] = pCurrentKeyframe;

        if (i == 0) {
          pResultKeyframes[1] = null;
        }
        else {
          pResultKeyframes[1] = pResultKeyframes[0];
          pResultKeyframes[0] = pKeyframes.get(i-1);
        }

        break;

      }
    }

    if (pResultKeyframes[0] == null) {
      pResultKeyframes[0] = pKeyframes.get(pKeyframes.size()-1);
      pResultKeyframes[1] = null;
    }

    return true;
  }*/


  /*
  //TO DO: Find out if this should be really done here. There is a different realization in the DoubleParameter 
  //       with the interpolated double class.
  public static int getLinearInterpolatedInt( double dTime0,
                                              int iValue0,
                                              double dTime1,
                                              int iValue1,
                                              double dCurrentTime) {

    double dInterpolatedValue = getLinearInterpolatedDouble(dTime0, iValue0, dTime1, iValue1, dCurrentTime);

    return (int)dInterpolatedValue;

  }


  public static float getLinearInterpolatedFloat( double dTime0,
                                                  float fValue0,
                                                  double dTime1,
                                                  float fValue1,
                                                  double dCurrentTime) {

    double dInterpolatedValue = getLinearInterpolatedDouble(dTime0, fValue0, dTime1, fValue1, dCurrentTime);

    return (float)dInterpolatedValue;

  }


  public static double getLinearInterpolatedDouble(   double dTime0,
                                                      double dValue0,
                                                      double dTime1,
                                                      double dValue1,
                                                      double dCurrentTime) {

    //1. make sure that dTime0 is before dTime1
    if (dTime0 > dTime1) {
      double dHelp = dTime0;
      dTime0 = dTime1;
      dTime1 = dHelp;

      dHelp = dValue0;
      dValue0 = dValue1;
      dValue1 = dHelp;
    }

    //2. check if the currentTime is within the given time slot
    if (dCurrentTime <= dTime0) return dValue0;
    if (dCurrentTime >= dTime1) return dValue1;


    //3. calculate the lambda parameter
    double dTotalTimeDiff = dTime1 - dTime0;
    double dLambda = (dCurrentTime - dTime0)/dTotalTimeDiff; //dLambda = 0, if dCurrentTime == dTime0

    //4. interpolate the value and return it
    double dInterpolatedValue = dValue0*(1 - dLambda) + dValue1*(dLambda);
    return dInterpolatedValue;

  }
  */


  /*public static <T extends IInterpolated> T getLinearInterpolatedObject(  double dTime0,
                                                                          T pObject0,
                                                                          double dTime1,
                                                                          T pObject1,
                                                                          double dCurrentTime) {
    //1. make a simple check
    if (pObject0 == null) return null;
    if (pObject1 == null) return null;

    //2. create an interpolated object
    return pObject0.getLinearInterpolatedObject(dTime0, dTime1, pObject1, dCurrentTime);
  }*/

}
