package standard;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.image.BufferedImage;



/**
 *  Interface that video object have to implement. 
 **/
public interface IVideo {

  ///////////////////////////////
  // GET METHODS               //
  ///////////////////////////////
  /**
   *  Returns the number of frames in the image sequence.
   *
   *  @return The number of frames in the image sequence as integer.
   */
  public int getFrameNumber();
  
  
  /**
   *  Returns the frame of the image sequence with the given frameID.
   *
   *  @param iFrameID ID of the frame that should be returned as integer. Valid values are from 0 to frameNumber - 1.
   *
   *  @return The frame of the image sequence with the given frameID as BufferedImage object or null if an error occurred.
   */
  public BufferedImage getFrame(int iFrameID);

  
  

}