package transformation;

/*
 *  Copyright (C) 2014 Juergen Platzer 
 * 
 *  This file is part of JFXLibrary.
 *  
 *  JFXLibrary is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *  
 *  JFXLibrary is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *  
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with JFXLibrary.  If not, see <http://www.gnu.org/licenses/>.
 */

//standard imports
import java.util.Hashtable;

//library imports
import standard.EffectParameters;
import standard.IntegerParameter;
import standard.ColorParameter;
import standard.ObjectParameter;
import util.image.ExampleImageGenerator;

/**
 *   Encapsulates the parameters that can be specified for the ImageResize object. 
 **/
public class ImageResizeParameters extends EffectParameters {

  /*** PUBLIC PARAMETER KEYS ***/

  /*** KEY for the integer parameter: The width of the resized image in pixels.  ***/
  public static final String PARAM_INT_NEW_WIDTH = "NEW_WIDTH";
  
  /*** KEY for the color parameter: The height of the resized image in pixels.  ***/
  public static final String PARAM_INT_NEW_HEIGHT = "NEW_HEIGHT";
  
   /*** KEY for the object parameter: The image that should be resized. ***/
  public static final String PARAM_OBJECT_IMAGE = "IMAGE";

    
   /***
   *   Creates a default parameter set for the image resize effect. The following settings are used:
   *
   *   <ul>
   *     <li>The size of the resized image is set to 1024 x 768 pixels</li>
   *     <li>A default image is set. (TO DO: There should be a default image method that is used to create the default image!!!)</li>
   *   </ul>
   *
   **/
  public ImageResizeParameters() {

    //1. create the parameters and set them
    setIntegerParameters(createIntegerParameters());
    setObjectParameters(createObjectParameters());

  }
  
  
  ////////////////////////////////////////////
  // PRIVATE METHODS FOR PARAMETER CREATION //
  ////////////////////////////////////////////
  private Hashtable<String, IntegerParameter> createIntegerParameters() {
    //1. create the hashtable
    Hashtable<String, IntegerParameter> pParameters = new Hashtable<String, IntegerParameter>();

    //2. fill the hashtable
    pParameters.put(PARAM_INT_NEW_WIDTH, new IntegerParameter(1024, 1, 50000));
    pParameters.put(PARAM_INT_NEW_HEIGHT, new IntegerParameter(768, 1, 50000));
    
    //3. return the hashtable
    return pParameters;
  }
  
  
  private Hashtable<String, ObjectParameter> createObjectParameters() {
    //1. create the hashtable
    Hashtable<String, ObjectParameter> pParameters = new Hashtable<String, ObjectParameter>();

    //2. fill the hashtable
    ExampleImageGenerator pGenerator = new ExampleImageGenerator();
    pParameters.put(PARAM_OBJECT_IMAGE, new ObjectParameter(pGenerator.getImage()));
  
    //3. return the hashtable
    return pParameters;
  }
  
  
  
  
  
}
  
  
  
